/*
 * System.c
 *
 * Created on: May 25, 2023
 * Author: TLE
 */

#include <assert.h>
#include <stdint.h>
#include <string.h>

#include <ucos_ii.h>
#include <cpu_core.h>
#include <lib_str.h>

#include <HL_sys_vim.h>
#include <HL_sci.h>
#include <HL_gio.h>
#include <HL_rti.h>

#include <app_cfg.h>


static OS_STK m_taskSystemStack[TASK_SYSTEM_STACK_SIZE];

OS_EVENT* gp_txGuard2 = 0;
OS_EVENT* gp_txGuard3 = 0;


/**
 * @brief
 * @details
 *
 */
void TaskSystem(void* data) {
    uint8_t error;

	gioInit();
	sciInit();

    rtiInit();
    rtiEnableNotification(rtiREG1, rtiNOTIFICATION_COMPARE1);
    rtiStartCounter(rtiREG1, rtiCOUNTER_BLOCK1);

#if 0
    Mem_Init();
  	Math_Init();
#endif

#if (OS_TASK_STAT_EN > 0u)
    OSStatInit();
#endif

#ifdef CPU_CFG_INT_DIS_MEAS_EN
    CPU_IntDisMeasMaxCurReset();
#endif

    gp_txGuard2 = OSSemCreate(1);
    gp_txGuard3 = OSSemCreate(1);

	while (1) {

#if 1	// This does not work
        OSSemPend(gp_txGuard2, 0, &error);
        assert(error == OS_ERR_NONE);
        sciSend(sciREG2, 10, "ABCDEFGHIJ");
#endif

#if 0	// This works
        OSSemPend(gp_txGuard3, 0, &error);
        assert(error == OS_ERR_NONE);
        sciSend(sciREG3, 10, "0123456789");
#endif

        OSTimeDly(2); // 1 tick = 10 ms
	}
}

/**
 * @brief
 * @details
 *
 */
void System_Initialize(void) {
	uint8_t error;
#if (CPU_CFG_NAME_EN == DEF_ENABLED)
    CPU_ERR errorCPU;
#endif

#if 1
    CPU_Init();
#endif

#if (CPU_CFG_NAME_EN == DEF_ENABLED)
    CPU_NameSet((char const*)"RM57L843", (CPU_ERR*)&errorCPU);
#endif

	OSInit();

	// Create System task
	error = OSTaskCreateExt(TaskSystem,
							0,
							&m_taskSystemStack[TASK_SYSTEM_STACK_SIZE - 1],
							TASK_SYSTEM_PRIORITY,
							TASK_SYSTEM_PRIORITY,
							&m_taskSystemStack[0],
							TASK_SYSTEM_STACK_SIZE,
							0,
							(OS_TASK_OPT_STK_CHK | OS_TASK_OPT_STK_CLR)); ///< Create System task
	assert(error == OS_ERR_NONE);

#if (OS_TASK_NAME_EN > 0u)
    OSTaskNameSet(TASK_SYSTEM_PRIORITY, (uint8_t*)"System Task", &error);
#endif
}

/**
 * @brief Start multi-tasking
 * @details
 */
void System_Run(void) {
	OSStart();
}


