/** @file HL_sys_main.c 
*   @brief Application main file
*   @date 11-Dec-2018
*   @version 04.07.01
*
*   This file contains an empty main function,
*   which can be used for the application.
*/

/* 
* Copyright (C) 2009-2018 Texas Instruments Incorporated - www.ti.com  
* 
* 
*  Redistribution and use in source and binary forms, with or without 
*  modification, are permitted provided that the following conditions 
*  are met:
*
*    Redistributions of source code must retain the above copyright 
*    notice, this list of conditions and the following disclaimer.
*
*    Redistributions in binary form must reproduce the above copyright
*    notice, this list of conditions and the following disclaimer in the 
*    documentation and/or other materials provided with the   
*    distribution.
*
*    Neither the name of Texas Instruments Incorporated nor the names of
*    its contributors may be used to endorse or promote products derived
*    from this software without specific prior written permission.
*
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
*  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
*  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
*  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
*  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
*  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
*  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
*  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
*  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
*  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
*  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
*/


/* USER CODE BEGIN (0) */
/* USER CODE END */

/* Include Files */

#include "HL_sys_common.h"

/* USER CODE BEGIN (1) */
#include <os.h>
#include <cpu_core.h>

#include <HL_sys_vim.h>

#include <System.h>
/* USER CODE END */

/** @fn void main(void)
*   @brief Application main function
*   @note This function is empty by default.
*
*   This function is called after startup.
*   The user can use this function to implement the application.
*/

/* USER CODE BEGIN (2) */
#define  VIM_RAM_TBL_ADDR_BASE       (CPU_ADDR)(0xFFF82000u)
/* USER CODE END */

int main(void)
{
/* USER CODE BEGIN (3) */
	System_Initialize();
	System_Run();
/* USER CODE END */

    return 0;
}


/* USER CODE BEGIN (4) */
/*
*********************************************************************************************************
*                                          ARM INTERRUPT/EXCEPTION HANDLER
*
* Description : Handle ARM exceptions.
*
* Argument(s) : except_type     ARM exception type identifier:
*
*                                  OS_CPU_ARM_EXCEPT_RESET             0x00
*                                  OS_CPU_ARM_EXCEPT_UNDEF_INSTR       0x01
*                                  OS_CPU_ARM_EXCEPT_SWI               0x02
*                                  OS_CPU_ARM_EXCEPT_ABORT_PREFETCH    0x03
*                                  OS_CPU_ARM_EXCEPT_ABORT_DATA        0x04
*                                  OS_CPU_ARM_EXCEPT_RSVD              0x05
*                                  OS_CPU_ARM_EXCEPT_IRQ               0x06
*                                  OS_CPU_ARM_EXCEPT_FIQ               0x07
*
* Return(s)   : none.
*
* Caller(s)   : OS_CPU_ARM_ExceptHndlr(), which is declared in os_cpu_a.s.
*
* Note(s)     : (1) Only OS_CPU_ARM_EXCEPT_IRQ exceptions handler is implemented. For the rest of the
*                   exception a infinite loop is implemented for debuging pruposes. This behavior should
*                   be replaced with another behavior (reboot, etc).
*
*               (2) OS_CPU_ARM_EXCEPT_FIQ implemented in Cortex-R5F is Non-maskable Fast Interrupt(NMFI).
*                   Once FIQ is enabled (by clearing F bit in CPSR, it can NOT be disabled by setting F bit
*                   in CPSR. Only a reset or an FIQ will be able to set the F bit in CPSR. Therefore, when
*                   using FIQ interrupts make sure that there are NO OS services calls being done from its
*                   ISRs because FIQ interrupt must be NON-KERNEL aware.
*********************************************************************************************************
*/

void  OS_CPU_ExceptHndlr (CPU_INT32U  except_type)
{
    CPU_FNCT_VOID   isr_fnct;
    CPU_REG32      *p_vim_tbl;
    CPU_INT32U      ch_ix;
    CPU_INT32U      pend_reg_0;
    CPU_INT32U      pend_reg_1;
    CPU_INT32U      pend_reg_2;
    CPU_INT32U      pend_reg_3;
    CPU_INT32U      pend_mask;


    switch (except_type) {
        case OS_CPU_ARM_EXCEPT_IRQ:
             ch_ix     = (CPU_INT32U     )vimREG->IRQINDEX;
             p_vim_tbl = (CPU_REG32     *)VIM_RAM_TBL_ADDR_BASE;
             isr_fnct  = (CPU_FNCT_VOID  )p_vim_tbl[ch_ix];

             if (isr_fnct != (CPU_FNCT_VOID)0) {
                 pend_reg_0 = vimREG->REQMASKCLR0;
                 pend_reg_1 = vimREG->REQMASKCLR1;
                 pend_reg_2 = vimREG->REQMASKCLR2;
                 pend_reg_3 = vimREG->REQMASKCLR3;
                 ch_ix--;

                 if (ch_ix <= 31) {
                     pend_mask = 0xFFFFFFFF << ch_ix;
                     vimREG->REQMASKCLR0 = pend_mask;
                     vimREG->REQMASKCLR1 = 0xFFFFFFFF;
                     vimREG->REQMASKCLR2 = 0xFFFFFFFF;
                     vimREG->REQMASKCLR3 = 0xFFFFFFFF;

                 } else if (ch_ix <= 63) {
                     pend_mask = 0xFFFFFFFF << (ch_ix - 32);
                     vimREG->REQMASKCLR1 = pend_mask;
                     vimREG->REQMASKCLR2 = 0xFFFFFFFF;
                     vimREG->REQMASKCLR3 = 0xFFFFFFFF;

                 } else if (ch_ix <= 95) {
                     pend_mask = 0xFFFFFFFF << (ch_ix - 64);
                     vimREG->REQMASKCLR2 = pend_mask;
                     vimREG->REQMASKCLR3 = 0xFFFFFFFF;

                 } else {
                     pend_mask = 0xFFFFFFFF << (ch_ix - 96);
                     vimREG->REQMASKCLR3 = pend_mask;
                 }

                 CPU_IntEn();
                 (*isr_fnct)();
                 CPU_IntDis();

                 vimREG->REQMASKSET0 = pend_reg_0;
                 vimREG->REQMASKSET1 = pend_reg_1;
                 vimREG->REQMASKSET2 = pend_reg_2;
                 vimREG->REQMASKSET3 = pend_reg_3;
             }
             break;


        case OS_CPU_ARM_EXCEPT_RESET:
        case OS_CPU_ARM_EXCEPT_UNDEF_INSTR:
        case OS_CPU_ARM_EXCEPT_SWI:
        case OS_CPU_ARM_EXCEPT_PREFETCH_ABORT:
        case OS_CPU_ARM_EXCEPT_DATA_ABORT:
        case OS_CPU_ARM_EXCEPT_ADDR_ABORT:
        case OS_CPU_ARM_EXCEPT_FIQ:
        default:
             while (DEF_TRUE) {
                 ;
             }
    }
}
/* USER CODE END */
